@@ -210,7 +210,8 @@ class _InstructionContainer:
                 # no need to copy dicts
                 return self._map_raw(_fn2gen1(fn))
             return self._map_raw(_fn2gen2(fn))
-
+        
+        # single row
         def _fn2gen(f):
             def gen(r):
                 x = f(r)
@@ -242,24 +243,16 @@ class _InstructionContainer:
         return self.map(updatefn)
 
     # pretty expensive for what it actually does
+    # but this version does not depend on the order of rows. 
     def rename(self, **kwargs):
-        pgen = self._genfns[-1]
+        kwargs_rev = {oldkey: newkey for newkey, oldkey in kwargs.items()}
 
-        def gen():
-            r0, rs = _spy(pgen())
-            oldcols = list(r0)
-            kwargs_rev = {old: new for new, old in kwargs.items()}
-            newcols = [kwargs_rev.get(old, old) for old in oldcols]
+        def renamefn(r):
+            yield {kwargs_rev.get(oldkey, oldkey): v
+                   for oldkey, v in r.items()}
 
-            for r in rs:
-                yield dict(zip(newcols, r.values()))
+        return self.map(renamefn)
 
-        self._genfns.append(gen)
-        self._insts.append({
-            'cmd': 'rename',
-            'kwargs': kwargs
-        })
-        return self
 
     def fold(self, **kwargs):
         gcols = []
