@@ -1,9 +1,9 @@
-import unittest
 import os
+import unittest
+from itertools import product
 from pathlib import Path
-from itertools import product, islice
-from tablemap import Conn
 
+from tablemap import Conn
 
 table1 = [
     {'col1': 'a', 'col2': 4},
@@ -25,6 +25,91 @@ table2 = [
 ]
 
 
+class TestMisc(unittest.TestCase):
+    def setUp(self):
+        self.dbfile = 'test/sample.db'
+
+    def tearDown(self):
+        if Path(self.dbfile).is_file():
+            os.remove(self.dbfile)
+
+    def test_index(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+
+        def cnt():
+            n = 0
+
+            def innerfn(rs):
+                nonlocal n
+                n += 1
+                for r in rs:
+                    r['n'] = n
+                    yield r
+            return innerfn
+
+        conn['t1_1'] = conn['t1'].by('col1').map(cnt())
+        self.assertEqual(
+            conn['t1_1'].list(), [{'col1': 'a', 'col2': 4, 'n': 1},
+                                  {'col1': 'a', 'col2': 5, 'n': 1},
+                                  {'col1': 'b', 'col2': 1, 'n': 2},
+                                  {'col1': 'c', 'col2': 3, 'n': 3},
+                                  {'col1': 'c', 'col2': 4, 'n': 3},
+                                  {'col1': 'c', 'col2': 7, 'n': 3},
+                                  {'col1': 'd', 'col2': 2, 'n': 4}])
+
+
+class TestFold(unittest.TestCase):
+    def setUp(self):
+        self.dbfile = 'test/sample.db'
+
+    def tearDown(self):
+        if Path(self.dbfile).is_file():
+            os.remove(self.dbfile)
+
+    def test_fold1(self):
+        conn = Conn(self.dbfile)
+
+        def minmax(col):
+            def fn(rs):
+                xs = [r[col] for r in rs]
+                return [min(xs), max(xs)]
+            return fn
+
+        conn['t1'] = table1
+        conn['t1_1'] = conn['t1'].by('col1').fold(
+            sum=lambda rs: sum(r['col2'] for r in rs),
+            mm=minmax('col2')
+        )
+
+        self.assertEqual(conn['t1_1'].list(),
+                         [{'col1': 'a', 'sum': 9, 'mm0': 4, 'mm1': 5},
+                          {'col1': 'b', 'sum': 1, 'mm0': 1, 'mm1': 1},
+                          {'col1': 'c', 'sum': 14, 'mm0': 3, 'mm1': 7},
+                          {'col1': 'd', 'sum': 2, 'mm0': 2, 'mm1': 2}])
+
+        conn['t1_1'] = conn['t1'].windowed(4, 3).fold(
+            sum=lambda rs: sum(r['col2'] for r in rs),
+            mm=minmax('col2')
+        )
+        self.assertEqual(conn['t1_1'].list(),
+                         [{'sum': 13, 'mm0': 1, 'mm1': 5},
+                          {'sum': 16, 'mm0': 2, 'mm1': 7}
+                          ])
+
+        conn['t1_1'] = conn['t1'].windowed(4, 5).fold(
+            sum=lambda rs: sum(r['col2'] for r in rs),
+            mm=minmax('col2')
+        ).rename(
+            mm1='mm0',
+            mm0='mm1'
+        )
+        self.assertEqual(conn['t1_1'].list(),
+                         [{'sum': 13, 'mm1': 1, 'mm0': 5},
+                          {'sum': 9, 'mm1': 2, 'mm0': 7}
+                          ])
+
+
 class TestGeneral(unittest.TestCase):
     def setUp(self):
         self.dbfile = 'test/sample.db'
@@ -135,15 +220,15 @@ class TestBy(unittest.TestCase):
         self.assertEqual(conn['t1_1_double'].list(),
                          conn['t1_1_double2'].list())
 
-    def test_itertools(self):
+    def test_islice(self):
 
         conn = Conn(self.dbfile)
         conn['t1'] = table1
         conn['t2'] = table2
 
-        conn['t1_1'] = islice(conn['t1'].iter(), 0, 3)
-
-        self.assertEqual(len(conn['t1_1'].list()), 3)
+        self.assertEqual(len(conn['t1'].islice(3).list()), 3)
+        self.assertEqual(len(conn['t1'].islice(3, None).list()), 4)
+        self.assertEqual(len(conn['t1'].islice(0, None, 3).list()), 3)
 
         xs = conn['t1'].iter()
         ys = conn['t2'].iter()
@@ -156,6 +241,50 @@ class TestBy(unittest.TestCase):
 
         self.assertEqual(n3, n1 * n2)
 
+    def test_takewhile(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+
+        self.assertEqual(len(conn['t1']
+                             .takewhile(lambda r: r['col2'] > 1).list()), 2)
+
+        self.assertEqual(len(conn['t1']
+                             .order('col2 desc')
+                             .takewhile(lambda r: r['col2'] > 1).list()), 6)
+
+    def test_dropwhile(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+
+        self.assertEqual(len(conn['t1']
+                             .dropwhile(lambda r: r['col2'] > 1).list()), 5)
+
+        self.assertEqual(len(conn['t1']
+                             .order('col2')
+                             .dropwhile(lambda r: r['col2'] < 4).list()), 4)
+
+    def test_windowed(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+
+        def add1(rs):
+            for r in rs:
+                r['col2'] += 1
+                yield r
+
+        self.assertEqual(conn['t1'].windowed(4, 2).map(add1).list(),
+                         [{'col1': 'a', 'col2': 5},
+                          {'col1': 'a', 'col2': 6},
+                          {'col1': 'b', 'col2': 2},
+                          {'col1': 'c', 'col2': 4},
+                          {'col1': 'b', 'col2': 2},
+                          {'col1': 'c', 'col2': 4},
+                          {'col1': 'c', 'col2': 5},
+                          {'col1': 'c', 'col2': 8},
+                          {'col1': 'c', 'col2': 5},
+                          {'col1': 'c', 'col2': 8},
+                          {'col1': 'd', 'col2': 3}])
+
 
 class TestDerivs(unittest.TestCase):
     def setUp(self):
@@ -173,10 +302,6 @@ class TestDerivs(unittest.TestCase):
 
         self.assertEqual(len(conn['t1_1'].list()), 5)
 
-        conn['t1_2'] = conn['t1'].by('col1').filter(lambda rs: len(rs) > 1)
-
-        self.assertEqual(len(conn['t1_2'].list()), 5)
-
     def test_update(self):
         conn = Conn(self.dbfile)
         conn['t1'] = table1
@@ -223,29 +348,42 @@ class TestDerivs(unittest.TestCase):
         conn['t1_4'] = conn['t1'].by('col1').full_join(conn['t2'].by('col1'))
 
         self.assertEqual(conn['t1_4'].list(), [
-            {'col1': 'a', 'col2': 4, 'col3': None},
-            {'col1': 'a', 'col2': 5, 'col3': None},
+            {'col1': 'a', 'col2': 4, 'col3': ''},
+            {'col1': 'a', 'col2': 5, 'col3': ''},
             {'col1': 'b', 'col2': 1, 'col3': 1},
-            {'col1': 'c', 'col2': 3, 'col3': None},
-            {'col1': 'c', 'col2': 4, 'col3': None},
-            {'col1': 'c', 'col2': 7, 'col3': None},
+            {'col1': 'c', 'col2': 3, 'col3': ''},
+            {'col1': 'c', 'col2': 4, 'col3': ''},
+            {'col1': 'c', 'col2': 7, 'col3': ''},
             {'col1': 'd', 'col2': 2, 'col3': 3},
             {'col1': 'd', 'col2': 2, 'col3': 2},
-            {'col1': 'e', 'col2': None, 'col3': 3},
-            {'col1': 'e', 'col2': None, 'col3': 4}]
+            {'col1': 'e', 'col2': '', 'col3': 3},
+            {'col1': 'e', 'col2': '', 'col3': 4}]
         )
 
-    def test_fold(self):
+    def test_join2(self):
         conn = Conn(self.dbfile)
         conn['t1'] = table1
+        conn['t2'] = table2
+
+        conn['t1_1'] = conn['t1'].concat([{'col1': 'd', 'col2': 9}, {
+            'col1': 'd', 'col2': 19}]).by('col1')\
+            .full_join(conn['t2'].by('col1'))
 
-        conn['t1_1'] = conn['t1'].by('col1').fold(
-            tot=lambda rs: sum(r['col2'] for r in rs))
         self.assertEqual(conn['t1_1'].list(),
-                         [{'col1': 'a', 'col2': 4, 'tot': 9},
-                          {'col1': 'b', 'col2': 1, 'tot': 1},
-                          {'col1': 'c', 'col2': 3, 'tot': 14},
-                          {'col1': 'd', 'col2': 2, 'tot': 2}])
+                         [{'col1': 'a', 'col2': 4, 'col3': ''},
+                          {'col1': 'a', 'col2': 5, 'col3': ''},
+                          {'col1': 'b', 'col2': 1, 'col3': 1},
+                          {'col1': 'c', 'col2': 3, 'col3': ''},
+                          {'col1': 'c', 'col2': 4, 'col3': ''},
+                          {'col1': 'c', 'col2': 7, 'col3': ''},
+                          {'col1': 'd', 'col2': 2, 'col3': 3},
+                          {'col1': 'd', 'col2': 2, 'col3': 2},
+                          {'col1': 'd', 'col2': 9, 'col3': 3},
+                          {'col1': 'd', 'col2': 9, 'col3': 2},
+                          {'col1': 'd', 'col2': 19, 'col3': 3},
+                          {'col1': 'd', 'col2': 19, 'col3': 2},
+                          {'col1': 'e', 'col2': '', 'col3': 3},
+                          {'col1': 'e', 'col2': '', 'col3': 4}])
 
     def test_distinct(self):
         conn = Conn(self.dbfile)
@@ -350,11 +488,6 @@ class TestReadMe(unittest.TestCase):
                          [{'col1': 'a', 'col2': 5, 'col3': 'a5'},
                           {'col1': 'a', 'col2': 6, 'col3': 'a6'}])
 
-        conn['t1_2'] = conn['t1'].by('col1').filter(lambda rs: len(rs) > 1)
-
-        self.assertEqual(conn['t1_2'].list(), [
-                         {'col1': 'a', 'col2': 4}, {'col1': 'a', 'col2': 5}])
-
         conn['t1_1'] = conn['t1'].update(col3=lambda r: r['col2'] + 1)\
             .deselect('col1', 'col2')
 
@@ -365,27 +498,19 @@ class TestReadMe(unittest.TestCase):
         self.assertEqual(conn['t1_1'].list(), [
                          {'col1': 'a', 'col2': 4}, {'col1': 'b', 'col2': 1}])
 
-        conn['t1_col2sum_groupby_col1'] = conn['t1'].by('col1')\
-            .fold(col2_sum=lambda rs: sum(r['col2'] for r in rs))\
-            .deselect('col2')
-
-        self.assertEqual(conn['t1_col2sum_groupby_col1'].list(), [
-                         {'col1': 'a', 'col2_sum': 9},
-                         {'col1': 'b', 'col2_sum': 1}])
-
         conn['t2'] = [
             {'col1': 'b', 'col3': -1},
             {'col1': 'c', 'col3': 3},
-            {'col1': 'b', 'col3': None},
+            {'col1': 'b', 'col3': ''},
         ]
 
         conn['t1_col3'] = conn['t1'].by('col1')\
             .full_join(conn['t2'].by('col1'))
 
         self.assertEqual(conn['t1_col3'].list(),
-                         [{'col1': 'a', 'col2': 4, 'col3': None},
-                         {'col1': 'a', 'col2': 5, 'col3': None},
+                         [{'col1': 'a', 'col2': 4, 'col3': ''},
+                         {'col1': 'a', 'col2': 5, 'col3': ''},
                          {'col1': 'b', 'col2': 1, 'col3': -1},
-                         {'col1': 'b', 'col2': 1, 'col3': None},
-                          {'col1': 'c', 'col2': None, 'col3': 3}]
+                         {'col1': 'b', 'col2': 1, 'col3': ''},
+                          {'col1': 'c', 'col2': '', 'col3': 3}]
                          )
